@@ -374,7 +374,7 @@ nautilus_get_vfs_method_display_name (char *method)
 	} else if (g_ascii_strcasecmp (method, "themes") == 0 ) {
 		return _("Themes");
 	} else if (g_ascii_strcasecmp (method, "burn") == 0 ) {
-		return _("CD Creator");
+		return _("CD/DVD Creator");
 	} else if (g_ascii_strcasecmp (method, "smb") == 0 ) {
 		return _("Windows Network");
 	} else if (g_ascii_strcasecmp (method, "dns-sd") == 0 ) {
