@@ -59,6 +59,14 @@ G_BEGIN_DECLS
 #define NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES  		"/desktop/gnome/file_views/show_hidden_files"
 #define NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES  		"/desktop/gnome/file_views/show_backup_files"
 #define NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS			"preferences/show_special_flags"
+#define NAUTILUS_PREFERENCES_DATE_FORMAT			"preferences/date_format"
+
+typedef enum
+{
+	NAUTILUS_DATE_FORMAT_LOCALE,
+	NAUTILUS_DATE_FORMAT_ISO,
+	NAUTILUS_DATE_FORMAT_INFORMAL
+} NautilusDateFormat;
 
 /* Sidebar panels  */
 #define NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES         "sidebar_panels/tree/show_only_directories"
@@ -168,9 +176,6 @@ typedef enum
 	NAUTILUS_SIMPLE_SEARCH_BAR
 } NautilusSearchBarMode;
 
-/* Gnome session management */
-#define NAUTILUS_PREFERENCES_ADD_TO_SESSION		   "preferences/add_to_session"
-
 #define NAUTILUS_PREFERENCES_DESKTOP_HOME_VISIBLE          "desktop/home_icon_visible"
 #define NAUTILUS_PREFERENCES_DESKTOP_HOME_NAME             "desktop/home_icon_name"
 #define NAUTILUS_PREFERENCES_DESKTOP_COMPUTER_VISIBLE      "desktop/computer_icon_visible"
@@ -180,9 +185,7 @@ typedef enum
 #define NAUTILUS_PREFERENCES_DESKTOP_VOLUMES_VISIBLE	   "desktop/volumes_visible"
 
 void nautilus_global_preferences_init                      (void);
-void nautilus_global_preferences_init_with_folder_browsing (void);
-void nautilus_global_preferences_set_default_folder_viewer (const char *iid);
-
+char *nautilus_global_preferences_get_default_folder_viewer_preference_as_iid (void);
 G_END_DECLS
 
 #endif /* NAUTILUS_GLOBAL_PREFERENCES_H */
